// Code generated by MockGen. DO NOT EDIT.
// Source: github.com/confluentinc/cli/v3/pkg/flink/types (interfaces: OutputControllerInterface)

// Package mock is a generated GoMock package.
package mock

import (
	reflect "reflect"

	gomock "github.com/golang/mock/gomock"
)

// MockOutputControllerInterface is a mock of OutputControllerInterface interface.
type MockOutputControllerInterface struct {
	ctrl     *gomock.Controller
	recorder *MockOutputControllerInterfaceMockRecorder
}

// MockOutputControllerInterfaceMockRecorder is the mock recorder for MockOutputControllerInterface.
type MockOutputControllerInterfaceMockRecorder struct {
	mock *MockOutputControllerInterface
}

// NewMockOutputControllerInterface creates a new mock instance.
func NewMockOutputControllerInterface(ctrl *gomock.Controller) *MockOutputControllerInterface {
	mock := &MockOutputControllerInterface{ctrl: ctrl}
	mock.recorder = &MockOutputControllerInterfaceMockRecorder{mock}
	return mock
}

// EXPECT returns an object that allows the caller to indicate expected use.
func (m *MockOutputControllerInterface) EXPECT() *MockOutputControllerInterfaceMockRecorder {
	return m.recorder
}

// VisualizeResults mocks base method.
func (m *MockOutputControllerInterface) VisualizeResults() {
	m.ctrl.T.Helper()
	m.ctrl.Call(m, "VisualizeResults")
}

// VisualizeResults indicates an expected call of VisualizeResults.
func (mr *MockOutputControllerInterfaceMockRecorder) VisualizeResults() *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "VisualizeResults", reflect.TypeOf((*MockOutputControllerInterface)(nil).VisualizeResults))
}
